/*
KDE Personal Web Server - Allow you to easily share files over http
Copyright (C) 2010 by Vitor Boschi da Silva (vitorboschi@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mainwindow.h"
#include "contentmodel.h"
#include "webcontent_file.h"

#include <KApplication>
#include <KLocalizedString>
#include <KActionCollection>
#include <KStandardAction>
#include <KAction>
#include <KFileDialog>
#include <KIO/NetAccess>
#include <KUrl>
#include <KMessageBox>
#include <KInputDialog>

#include <QNetworkInterface>

MainWindow::MainWindow(QWidget *parent) : KXmlGuiWindow(parent)
{

	layout = new KVBox(this);
	setCentralWidget(layout);
	bindAddress = new KComboBox(true, layout);
	foreach(QNetworkInterface interface, QNetworkInterface::allInterfaces())
	{
		if (interface.flags().testFlag(QNetworkInterface::IsRunning)) {
			foreach (QNetworkAddressEntry entry, interface.addressEntries())
			{
			if ( interface.hardwareAddress() != "00:00:00:00:00:00" &&     entry.ip().toString().contains("."))
				bindAddress->addItem("http://" + entry.ip().toString() + ":8080",entry.ip().toString());
			}
		}
	}
	
	
	resourcesView = new QTreeView(layout);
	logList = new KListWidget(layout);
	logList->addItem(i18n("Application started"));
	
	resourcesView->setModel(new ContentModel(resourcesView));
	
	//TODO: create a proper API for this
	root = static_cast<WebContent_VirtualFolder*>(static_cast<ContentModel*>(resourcesView->model())->rootItem);
	
	m_contentModel = static_cast<ContentModel*>(resourcesView->model());
		
	daemon.setRootContent(root);
	
	setupActions();

	if (daemon.listen(QHostAddress::Any, 8080)) {
		logList->addItem(i18n("Daemon started"));
	}
}


void MainWindow::setupActions()
{
	KAction* shareAction = new KAction(this);
	KAction* shareFolderAction = new KAction(this);
	KAction* createVFolderAction = new KAction(this);
	KAction* removeAction = new KAction(this);
	
	
	shareAction->setText(i18n("Share File"));
	shareAction->setIcon(KIcon("archive-insert"));
	actionCollection()->addAction("shareFile", shareAction);
	connect(shareAction, SIGNAL( triggered(bool) ), 
			  this, SLOT( slotShareFile() ) );
			  
	shareFolderAction->setText(i18n("Share Folder"));
	shareFolderAction->setIcon(KIcon("archive-insert-directory"));
	actionCollection()->addAction("shareFolder", shareFolderAction);
	connect(shareFolderAction, SIGNAL( triggered(bool) ), 
			  this, SLOT( slotShareFolder() ) );

	
	createVFolderAction->setText(i18n("Create Virtual Folder"));
	createVFolderAction->setIcon(KIcon("folder-new"));
	actionCollection()->addAction("createVFolder", createVFolderAction);
	connect(createVFolderAction, SIGNAL( triggered(bool) ), 
			  this, SLOT( slotCreateVFolder()));
	
	removeAction->setText(i18n("Stop sharing item"));
	removeAction->setIcon(KIcon("list-remove"));
	actionCollection()->addAction("remove", removeAction);

	KStandardAction::quit(kapp, SLOT(quit()), actionCollection());

	setupGUI();
}

void MainWindow::slotShareFile()
{
	KUrl url;
	
	url = KFileDialog::getOpenFileName(QString(), i18n("*.*|All files"), this, i18n("Share File") );
	if (url == "") {
		return;
	}
	//TODO: Check for permissions before trying to open
	WebContent_File* f = new WebContent_File(url.fileName(), url.url());
	m_contentModel->addContent(f, resourcesView->currentIndex());
	resourcesView->expandAll();
	
	qDebug() << "Share file";
}

void MainWindow::slotCreateVFolder()
{
	QString id;
	
	id = KInputDialog::getText(i18n("Create Virtual Folder"), i18n("Folder name:"));
	if (id != "") {
		//TODO: Sanitize user input
		WebContent_VirtualFolder* vFolder = new WebContent_VirtualFolder(id);
		m_contentModel->addContent(vFolder, resourcesView->currentIndex());
		resourcesView->expandAll();
	}
}

void MainWindow::slotShareFolder()
{
	QString folder;
	folder = KFileDialog::getExistingDirectory(KUrl(), this, i18n("Share folder"));
	if (folder.isEmpty()) {
		return;
	}
	
	QModelIndex parent = resourcesView->currentIndex();
	m_contentModel->addContent(shareFolder(folder, NULL), resourcesView->currentIndex());
	resourcesView->expandAll();
}

WebContent_VirtualFolder* MainWindow::shareFolder(QString folder, WebContent_VirtualFolder* parent)
{
	QDir dir(folder);
	QString entry;
	WebContent_File* f;
	
	//add the virtual folder for this folder
	WebContent_VirtualFolder* vFolder = new WebContent_VirtualFolder(dir.dirName());

	//add files
	foreach(entry,dir.entryList(QDir::Files | QDir::Readable)) {
		qDebug() << "Sharing file " << entry;
		f = new WebContent_File(entry, dir.absoluteFilePath(entry));
		vFolder->addContent(f);
	}
	
	//recursively add folders
	foreach(entry,dir.entryList(QDir::Dirs | QDir::Readable | QDir::NoDotAndDotDot)) {
		qDebug() << "Sharing folder " << entry;
		vFolder->addContent(shareFolder(dir.absoluteFilePath(entry), vFolder));
	}
	return vFolder;
}
